# Generated by Django 2.2.16 on 2022-04-01 15:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("swh_web_add_forge_now", "0004_rename_tables"),
    ]

    operations = [
        migrations.AddField(
            model_name="requesthistory",
            name="message_source",
            field=models.BinaryField(null=True),
        ),
        migrations.AlterField(
            model_name="requesthistory",
            name="actor_role",
            field=models.TextField(
                choices=[
                    ("MODERATOR", "moderator"),
                    ("SUBMITTER", "submitter"),
                    ("FORGE_ADMIN", "forge admin"),
                    ("EMAIL", "email"),
                ]
            ),
        ),
    ]
