.swh-coverage{border:none;overflow:visible;padding-top:.3rem}.swh-coverage a{text-decoration:none}.swh-coverage-col{padding-left:10px;padding-right:10px}.swh-coverage-header{padding-bottom:0;padding-top:0}.swh-coverage-logo{display:block;font-family:"object-fit: contain;";height:50px;margin-left:auto;margin-right:auto;-o-object-fit:contain;object-fit:contain;width:100%}.swh-coverage-iframe{border:none;height:320px;width:100%}.swh-coverage-chevron{position:absolute;right:0}.swh-coverage .card-header .mdi{-webkit-transition:-webkit-transform .3s ease-in-out;transition:-webkit-transform .3s ease-in-out;-o-transition:-o-transform .3s ease-in-out;-moz-transition:transform .3s ease-in-out,-moz-transform .3s ease-in-out;transition:transform .3s ease-in-out;transition:transform .3s ease-in-out,-webkit-transform .3s ease-in-out,-moz-transform .3s ease-in-out,-o-transform .3s ease-in-out}.swh-coverage .card-header .collapsed .mdi{-webkit-transform:rotate(90deg);-moz-transform:rotate(90deg);-o-transform:rotate(90deg);transform:rotate(90deg)}.swh-coverage-info-body{scrollbar-width:thin;max-height:150px;overflow-x:hidden;overflow-y:auto;padding:0}.swh-coverage-focus{border:2px solid #e20026!important}.swh-coverage-info-body::-webkit-scrollbar{width:4px}.swh-coverage-info-body::-webkit-scrollbar-track{background:#eff0f1}.swh-coverage-info-body::-webkit-scrollbar-thumb{background:#909396}
/*# sourceMappingURL=archive_coverage.243fd0a38715dd7ff3e5.css.map */